package org.devcake.groovy.remoteconsole

import groovy.xml.MarkupBuilder

/**
 * Created by IntelliJ IDEA.
 * User: Admin
 * Date: 29.08.2010
 * Time: 22:53:29
 * To change this template use File | Settings | File Templates.
 */
class RemoteConsoleClient {
    public static final String DEFAULT_HOST = 'localhost'
    public static final int DEFAULT_PORT = 7777

    String host
    Integer port
    ScriptExecutionProcessor executionProcessor

    boolean shareSameBinding = true
    String encoding = System.getProperty("file.encoding")

    private boolean running
    private Socket clientSocket

    RemoteConsoleClient(ScriptExecutionProcessor executionProcessor) {
        this(null, null, executionProcessor)
    }
    RemoteConsoleClient(String host, Integer port, ScriptExecutionProcessor executionProcessor) {
        this.host = host;
        this.port = port;
        this.executionProcessor = executionProcessor;
    }

    void execute(String scriptText) {
        execute([:], scriptText)
    }

    void execute(Map<String,String>  options, String scriptText) {
        execute(options, scriptText, scriptText)
    }

    void execute(Map<String,String> configurationOptions, String fullScriptText, String selectedScriptText) {
        running = true
        try {
            doExecute(configurationOptions, fullScriptText, selectedScriptText)
        } finally {
            running = false
        }
    }

    /**
     * Returns user's UID in form "hostname-username"
     */
    private String getUserUID() {
        def hostName = java.net.InetAddress.getLocalHost().getHostName()
        def userName = System.getProperty("user.name")
        return "$hostName-$userName"
    }

    private def doExecute(Map<String, String> configurationOptions, String fullScriptText, String selectedScriptText) {
        def host = this.host ?: configurationOptions.host ?: DEFAULT_HOST
        def port = this.port ?: configurationOptions.port ?: DEFAULT_PORT
        clientSocket = new Socket(host, port)
        clientSocket.withStreams { inputStream, outputStream ->
            def outWriter = new PrintWriter(new OutputStreamWriter(outputStream, encoding), true)
            def inReader = new BufferedReader(new InputStreamReader(inputStream, encoding))
            def xml = new MarkupBuilder(outWriter)
            def resultReturnSeparator = '------------------' + new Date() + '---------------------------------'
            def resultErrorSeparator = 'EEEEEEEEEEEEEEEEEE' + new Date() + 'EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE'
            xml.execute {
                options {
                    configurationOptions.each {k, v ->
                        option(name: k, value: v)
                    }
                    if (shareSameBinding && !configurationOptions.binding) {
                        option(name:'binding', value:getUserUID())
                    }
                }
                fullScript(fullScriptText)
                selectedScript(selectedScriptText)
                errorSeparator(resultErrorSeparator)
                returnSeparator(resultReturnSeparator)
            }
            outWriter.println('\n')
            def inputLine
            executionProcessor.startOutput()
            while ((inputLine = inReader.readLine()) && !(inputLine in [null, resultReturnSeparator, resultErrorSeparator])) {
                if (inputLine.endsWith(RemoteConsoleServer.END_LINE_OUTPUT_SUFFIX)) {
                    executionProcessor.processOutputLine(inputLine.replace(RemoteConsoleServer.END_LINE_OUTPUT_SUFFIX, ''))
                } else if (inputLine.endsWith(RemoteConsoleServer.END_LINE_ERROR_SUFFIX)) {
                    executionProcessor.processOutputErrorLine(inputLine.replace(RemoteConsoleServer.END_LINE_ERROR_SUFFIX, ''))
                }
            }
            executionProcessor.endOutput()
            if (inputLine == resultErrorSeparator) {
                executionProcessor.startError()
                while ((inputLine = inReader.readLine()) && !(inputLine in [null, resultReturnSeparator])) {
                    executionProcessor.processErrorLine(inputLine)
                }
                executionProcessor.endError()
            } else {
                executionProcessor.startResult()
                while ((inputLine = inReader.readLine()) != null) {
                    executionProcessor.processResultLine(inputLine)
                }
                executionProcessor.endResult()
            }
        }
        clientSocket.close()
    }

    public boolean isRunning() { running }

    void stop() {
        clientSocket.close()
        running = false
    }

    /**
     * Parses script text and returns script options, that are defined in first line.
     * Uses host = DEFAULT_HOST and port = DEFAULT_PORT, if these values are not defined
     */
    public static def getScriptOptions(String scriptText) {
        def firstLine = scriptText.trim().readLines()[0].trim()
        if (firstLine.replaceAll(/\s/, '').startsWith('//options:')) {
            def code = firstLine.replaceFirst('\\s*//\\s*options\\s*:\\s*', '')
            return Eval.me(code)
        } else {
            return [:]
        }
    }

    public static String stackTraceToString(Throwable e) {
        new StringWriter().with {
            def pw = new PrintWriter(it, true)
            e.printStackTrace(pw)
            pw.flush()
            it.toString()
        }
    }

}
